#pragma once

#include "ofMain.h"

//uuid
#include "random_id.h"

#include "ofxMsdfgen.h"

namespace ofxVariableLab {

class Layer {
    protected:
        Layer();
    public:
        ~Layer();
        enum TransformOrigin {
            TOP_LEFT,
            TOP_RIGHT,
            CENTER,
            BOTTOM_LEFT,
            BOTTOM_RIGHT
        };
        struct Props {
            float x = 0;
            float y = 0;
            float rotation = 0;
            float wght = 100;
            float fontSize_px = 24;
            float * color[4];
            TransformOrigin transformOrigin = TransformOrigin::CENTER;
            bool mirror_x = false;
            float mirror_x_distance = 0;
            bool mirror_y = false;
            float mirror_y_distance = 0;
            float letterDelay = 0;
            string text = "quickmonkeythedogASYAHELLO";
        };
        struct Settings {
            uint32_t maxBufferSize = 100;
        };

        void update();
        virtual void draw(glm::vec3 position = glm::vec3(0, 0, 0)) const = 0;
        void setProps(const Props & props);
        //void setProps(float _x,
        //float _y,
        //float _rotation,
        //float _wght,
        //float _fontSize_px,
        //float * _color[4],
        //int _transformOrigin,
        //bool _mirror_x,
        //float _mirror_x_distance,
        //bool _mirror_y,
        //float _mirror_y_distance,
        //float _letterDelay,
        //string _text);
        void clearPropsBuffer();
        void setId(const string & id);
        const string & getId();

        Settings settings;
        std::deque <Props> propsBuffer;
        shared_ptr <ofShader> shader;
        /// \brief are props updated but not drawn yet
        bool isDirty = true;
        /// \brief hashed id
        string id;
};

class MsdfLayer : public Layer {
    public:
        MsdfLayer();
        ~MsdfLayer();
        void draw(glm::vec3 position = glm::vec3(0, 0, 0)) const override;
        shared_ptr <ofxMsdfgen::Atlas> atlas;
};

}
